@@ -469,60 +469,3 @@ def is_periodic(state):
         return False
     else:
         return False
-
-
-if __name__ == "__main__":
-    import os
-    import matplotlib.pyplot as plt
-
-
-    def plot_mesh_res_for_vs_py(mesh_res_for_name: str, mesh_res_py_name: str):
-        """
-           plot the mesh points in fortran mesh vs python mesh.
-           """
-        xvals, yvals, rvals = [], [], []
-
-        with open(str(mesh_res_for_name), "r") as f:
-            fig, axs = plt.subplots(2, figsize=(7, 6.5))
-            for idx, line in enumerate(f):
-                if idx >= 1:
-                    for word in line.split()[1:2]:
-                        xvals.append(float(word))
-                    for word in line.split()[2:3]:
-                        yvals.append(float(word))
-                    for word in line.split()[3:4]:
-                        rvals.append(float(word))
-
-        # plot the legacy mesh
-        axs[0].scatter(xvals, yvals, label="FORTRAN mesh", s=1)
-        # plot the mesh ratio
-        axs[1].plot(xvals, rvals, label='FORTRAN Ratio')
-        axs[1].set_title("Ratio", y=1.3)
-
-        xvals, yvals, rvals = [], [], []
-
-        with open(str(mesh_res_py_name), "r") as f:
-            for idx, line in enumerate(f):
-                if idx >= 1:
-                    for word in line.split()[1:2]:
-                        xvals.append(float(word))
-                    for word in line.split()[2:3]:
-                        yvals.append(float(word))
-                    for word in line.split()[3:4]:
-                        rvals.append(float(word))
-
-        # plot the legacy mesh
-        axs[0].scatter(xvals, yvals, label="Py mesh", s=1)
-        # plot the mesh ratio
-        axs[1].plot(xvals, rvals, label=' Py Ratio')
-        axs[1].set_title("Ratio", y=1.3)
-
-        axs[0].legend(bbox_to_anchor=(0, 1.02, 1, 0.2), loc="lower left", mode="expand", ncol=3)
-        axs[1].legend(bbox_to_anchor=(0, 1.02, 1, 0.2), loc="lower left", mode="expand", ncol=1)
-        plt.tight_layout()
-        plt.show()
-
-
-    #write_mesh_res_file(os.path.join(os.getcwd(), "mesh_res_t.dat"),
-    #                    os.path.join(os.getcwd(), "tmp_mesh_t.dat"))
-    plot_mesh_res_for_vs_py("mesh_res_p.txt", "../../tests/fortran_mesh_res/mesh_res_p_1.dat")
