@@ -61,6 +61,8 @@ from .legacy_mesh import LegacyMesh
 from .legacy_mesh_segment import LegacyMeshSegment
 from .mesh_segment import MeshSegment
 
+EPSILON = 1 / numpy.power(10, 5)
+
 
 class Mesh:
     """
@@ -74,6 +76,7 @@ class Mesh:
         self.periodic = periodic
         self.phi_shift = numpy.float64(phi_shift)
         self.BG_RATIO = MeshSegment.DEFAULT_BG_REGION_RATIO
+        # self.FG_RATIO = MeshSegment.DEFAULT_FG_REGION_RATIO
         self.segment_list = numpy.array([MeshSegment(self.lower_bnd, self.upper_bnd)])
 
     def __str__(self):
@@ -87,6 +90,7 @@ class Mesh:
             'periodic': self.periodic,
             'phi_shift': self.phi_shift,
             'BG_RATIO': self.BG_RATIO,
+            # 'FG_RATIO': self.FG_RATIO,
             'segment_list': [segment.json_dict() for segment in self.segment_list.tolist()]
         }
 
@@ -167,8 +171,8 @@ class Mesh:
             segment_2 = MeshSegment(s0=0, s1=new_segment.s1 - self.upper_bnd, ds=new_segment.ds0,
                                     var_ds_ratio=new_segment.var_ds_ratio)
             index1 = self.get_mesh_segment_index(mesh_segment=segment_1)
-            index2 = self.get_mesh_segment_index(mesh_segment=segment_2)
             self.segment_list = numpy.insert(self.segment_list, index1, segment_1)
+            index2 = self.get_mesh_segment_index(mesh_segment=segment_2)
             self.segment_list = numpy.insert(self.segment_list, index2, segment_2)
 
         elif self.upper_bnd < new_segment.s0 and self.upper_bnd < new_segment.s1:
@@ -188,8 +192,8 @@ class Mesh:
             segment_2 = MeshSegment(s0=new_segment.s0 + self.upper_bnd, s1=self.upper_bnd, ds=new_segment.ds0,
                                     var_ds_ratio=new_segment.var_ds_ratio)
             index1 = self.get_mesh_segment_index(mesh_segment=segment_1)
-            index2 = self.get_mesh_segment_index(mesh_segment=segment_2)
             self.segment_list = numpy.insert(self.segment_list, index1, segment_1)
+            index2 = self.get_mesh_segment_index(mesh_segment=segment_2)
             self.segment_list = numpy.insert(self.segment_list, index2, segment_2)
 
         elif new_segment.s0 <= 0 and new_segment.s1 >= self.upper_bnd:
@@ -203,14 +207,12 @@ class Mesh:
         for index in range(beg_index, len(self.segment_list)):
             segment = self.segment_list[index]
             if mesh_segment.s0 < segment.s0:
-                break
+                return index
             elif mesh_segment.s0 == segment.s0:
                 if mesh_segment.s1 > segment.s1:
                     index += 1
-                break
-        else:
-            index = len(self.segment_list)
-        return index
+                return index
+        return len(self.segment_list)
 
     def remove_mesh_segment(self, index):
         """
@@ -458,32 +460,90 @@ class Mesh:
                     |  |==|  |
                     |==|  |==|
                     """
-
-                    new_segment = MeshSegment(
-                        0.5 * (segment.s0 + segment.s1), segment.s1,
-                        curr_ds, segment.var_ds_ratio)
-                    new_segment.set_ds(curr_ds, next_ds)
-                    segment.s1 = new_segment.s0
-                    segment.set_ds(prev_ds, curr_ds)
-
-                    index += 1
-                    self.segment_list = numpy.insert(
-                        self.segment_list, index, new_segment)
-                    ds_list.insert(index, curr_ds)
-                    end_index = len(self.segment_list) - 1
+                    dsh, sh = self._check_resolve_gap_mid_mesh(index, prev_ds, next_ds)
+                    if dsh - curr_ds <= EPSILON:
+                        self.resolve_gap_mid_mesh(segment, index, ds_begin=prev_ds, ds_end=next_ds)
+                        if end_index == len(self.segment_list) - 2:  # if gen_resolve_gap inserted a segment
+                            end_index = len(self.segment_list) - 1
+                            ds_list.insert(index + 1, prev_ds)
+                            ds_list[index + 1] = self.segment_list[index + 1].ds0
+                            index += 1
+                        else:
+                            ds_list[index] = self.segment_list[index + 1].ds0
+                    else:
+                        if segment.s0 < sh < segment.s1:
+                            new_segment = MeshSegment(
+                                sh, segment.s1, curr_ds, segment.var_ds_ratio)
+                            new_segment.set_ds(curr_ds, next_ds)
+                            segment.s1 = new_segment.s0
+                            segment.set_ds(prev_ds, curr_ds)
+                            index += 1
+                            self.segment_list = numpy.insert(
+                                self.segment_list, index, new_segment)
+                            ds_list.insert(index, curr_ds)
+                            end_index = len(self.segment_list) - 1
 
             index += 1
 
-    def build_legacy_mesh(self):
+    def _resolve_hat_mesh_segment(self, index):
+        """ This function resolves an adjusted mesh segment (step #2) where we have this pattern:
+                    |  |  |  |      |  |   == |  |
+                    |  |==|  | ---> |  |  //\\|  |
+                    |==|  |==|      |==|//  \\|==|
+
+        """
+
+        segment = self.segment_list[index]
+
+        curr_ds = segment.ds0
+        prev_ds = self.segment_list[index - 1].ds1
+        next_ds = self.segment_list[index + 1].ds0
+
+        dsh, sh = self._check_resolve_gap_mid_mesh(index, prev_ds, next_ds)
+
+        if dsh - curr_ds <= numpy.sqrt(EPSILON):
+            self.resolve_gap_mid_mesh(segment, index, ds_begin=prev_ds, ds_end=next_ds)
+
+        else:
+            if segment.s0 < sh < segment.s1:
+                new_segment = MeshSegment(sh, segment.s1, curr_ds, segment.var_ds_ratio)
+                new_segment.set_ds(curr_ds, next_ds)
+                segment.s1 = new_segment.s0
+                segment.set_ds(prev_ds, curr_ds)
+                index += 1
+                self.segment_list = numpy.insert(self.segment_list, index, new_segment)
+
+    def _get_ds_begin_and_ds_end(self, num_beg, num_end):
+        """ return ds0 of segment begin and ds1 of segment end. This is called to check if periodic condition
+        is satisfied. """
+        segment_begin = self.segment_list[0]
+        segment_end = self.segment_list[-1]
+
+        ds_begin = segment_begin.ds0
+        ds_end = segment_end.ds1
+
+        if ds_begin == numpy.inf:
+            # this is a fixed length segment so compute ds0 using the legacy function.
+            ds_begin = segment_begin.get_ds0_using_num_points(num_beg)
+        if ds_end == numpy.inf:
+            # this is a fixed length segment so compute ds1 using the legacy function.
+            ds_end = segment_end.get_ds1_using_num_points(num_end)
+
+        return ds_begin, ds_end
+
+    def build_legacy_mesh(self, max_iterations=500):
         """
         Builds and returns the legacy mesh using constraints of this mesh.
         """
         legacy_mesh = LegacyMesh(self.lower_bnd, self.upper_bnd, self.periodic, self.phi_shift)
 
-        i = 0
-        while i < len(self.segment_list):
+        jj = 0  # the cumulative number of iterations.
+        kk = 0  # number of iterations stuck on fixing the periodicity.
+        ii = 0  # current mesh segment index.
 
-            segment = self.segment_list[i]
+        while ii < len(self.segment_list) and jj < max_iterations:
+
+            segment = self.segment_list[ii]
             legacy_mesh_segment_list = segment.get_legacy_mesh_segments(self.lower_bnd, self.upper_bnd)
 
             if legacy_mesh_segment_list[0].s0 == segment.s0 and \
@@ -493,10 +553,29 @@ class Mesh:
 
                 # Update the next segment ds0 if needed OR the previous segment ds1. Ensures a continuous mesh.
                 if len(legacy_mesh_segment_list) == 1:
-                    if self.update_next_segment(segment, i, legacy_segment):
-                        i = - 1
+                    if self.update_next_segment(segment, ii, legacy_segment):
+                        ii = - 1
                         legacy_mesh.segment_list = numpy.array([], dtype=LegacyMeshSegment)
 
+                if legacy_segment.s1 == self.upper_bnd and self.periodic:
+                    # check if periodicity requirement is satisfied, if not adjust either begin or end segment to be
+                    # fixed ds. Then, reiterate and solve for legacy mesh.
+                    # Note: if kk > 2, this is a sign that we get a round off error since legacy mesh computes the
+                    # number of segment points using np.ceil, this can cause the begin and end ds to be slightly off.
+                    # In that case, we return the current legacy mesh and exit the while loop.
+                    ds_begin, ds_end = self._get_ds_begin_and_ds_end(num_beg=legacy_mesh.segment_list[0].num_points,
+                                                                     num_end=legacy_mesh.segment_list[-1].num_points)
+                    if abs(ds_begin - ds_end) > EPSILON:
+                        if ds_begin > ds_end:
+                            self.segment_list[0].ds0 = ds_end
+                        else:
+                            self.segment_list[-1].ds1 = ds_begin
+                        if kk < 2:
+                            # this loop is caused by using np.ceil to find the legacy segment num_points.
+                            ii = -1
+                            legacy_mesh.segment_list = numpy.array([], dtype=LegacyMeshSegment)
+                    kk += 1
+
             else:
                 if segment == self.segment_list[-1] and \
                         segment.has_increasing_ds():
@@ -511,23 +590,29 @@ class Mesh:
                 if segment.has_increasing_ds() and segment.s1 != self.upper_bnd:
 
                     # Next segment
-                    adjacent_segment = self.segment_list[i + 1]
+                    adjacent_segment = self.segment_list[ii + 1]
 
                     if legacy_mesh_segment_list[-1].s1 >= adjacent_segment.s1:
                         passes_mult_segment = True
 
                     if not passes_mult_segment and \
                             (not adjacent_segment.is_fixed_length() or not adjacent_segment.has_increasing_ds()):
-                        # CASE: PASSES ONE SEGMENT
-                        # CASE: NOT FIXED LENGTH, NOT INCREASING DS
+                        # CASE: PASSES ONE SEGMENT.
+                        # CASE: NOT FIXED LENGTH, NOT INCREASING DS.
+                        # RECALCULATE MID HAT POINT.
+                        if segment.is_fixed_ds() and adjacent_segment.is_fixed_ds() \
+                                and adjacent_segment.has_decreasing_ds() and adjacent_segment.s1 != self.upper_bnd:
+                            # remove one of the mesh segments.
+                            self.segment_list[ii + 1].s0 = segment.s0
+                            self.segment_list[ii + 1].ds1 = adjacent_segment.ds0
+                            self.remove_mesh_segment(ii)
+                            # recalculate the peak between the two segments.
+                            self._resolve_hat_mesh_segment(index=ii + 1)
 
-                        if segment.has_increasing_ds():
+                        else:
                             segment.s1 = legacy_mesh_segment_list[-1].s1
                             adjacent_segment.s0 = legacy_mesh_segment_list[-1].s1
 
-                        else:
-                            segment.s0 = legacy_mesh_segment_list[-1].s0
-                            adjacent_segment.s1 = legacy_mesh_segment_list[-1].s0
                     else:
                         # CASE: SEGMENT CROSSES MULTIPLE REGIONS
                         # CASE: FIXED LENGTH AND HAS INCREASING DS
@@ -538,9 +623,9 @@ class Mesh:
 
                         if adjacent_segment.s1 != self.upper_bnd and adjacent_segment.is_fixed_length() \
                                 and adjacent_segment.has_increasing_ds():
-                            if self.segment_list[i + 2].is_fixed_length() and \
-                                    self.segment_list[i + 2].has_decreasing_ds():
-                                if self.segment_list[i + 2].s1 != self.upper_bnd:
+                            if self.segment_list[ii + 2].is_fixed_length() and \
+                                    self.segment_list[ii + 2].has_decreasing_ds():
+                                if self.segment_list[ii + 2].s1 != self.upper_bnd:
                                     ''''
                                         Undershoot case mid-mesh. Calculate new intersection point. 
                                         Calling resolve gap again. 
@@ -551,11 +636,11 @@ class Mesh:
 
                                     '''
                                     # print("Recalculate resolve gap.")
-                                    adjacent_segment.s1 = self.segment_list[i + 2].s1
-                                    adjacent_segment.var_ds_ratio = segment.DEFAULT_BG_REGION_RATIO
-                                    adjacent_segment.ds1 = self.segment_list[i + 2].ds1
-                                    self.remove_mesh_segment(i + 2)
-                                    self.resolve_gap_mid_mesh(adjacent_segment, i + 1, ds_begin=ds,
+                                    adjacent_segment.s1 = self.segment_list[ii + 2].s1
+                                    # adjacent_segment.var_ds_ratio = segment.DEFAULT_BG_REGION_RATIO
+                                    adjacent_segment.ds1 = self.segment_list[ii + 2].ds1
+                                    self.remove_mesh_segment(ii + 2)
+                                    self.resolve_gap_mid_mesh(adjacent_segment, ii + 1, ds_begin=ds,
                                                               ds_end=adjacent_segment.ds1)
 
                         if adjacent_segment.s1 == self.upper_bnd and self.periodic and \
@@ -579,7 +664,7 @@ class Mesh:
 
                 if segment.has_decreasing_ds() and segment.s0 != self.lower_bnd:
                     # Previous segment
-                    adjacent_segment = self.segment_list[i - 1]
+                    adjacent_segment = self.segment_list[ii - 1]
                     if legacy_mesh_segment_list[-1].s0 <= adjacent_segment.s0:
                         passes_mult_segment = True
 
@@ -587,10 +672,18 @@ class Mesh:
                             (not adjacent_segment.is_fixed_length() or not adjacent_segment.has_decreasing_ds()):
                         # CASE: PASSES ONE SEGMENT
                         # CASE: NOT FIXED LENGTH, NOT DECREASING DS
-                        if segment.has_increasing_ds():
-                            segment.s1 = legacy_mesh_segment_list[-1].s1
-                            adjacent_segment.s0 = legacy_mesh_segment_list[-1].s1
+                        # RECALCULATE MID HAT POINT.
+                        if segment.is_fixed_ds() and adjacent_segment.is_fixed_ds() \
+                                and adjacent_segment.has_increasing_ds() and adjacent_segment.s0 != self.lower_bnd:
+                            # remove one of the mesh segments.
+                            self.segment_list[ii - 1].s1 = segment.s1
+                            self.segment_list[ii - 1].ds0 = adjacent_segment.ds1
+                            self.remove_mesh_segment(ii)
+                            # recalculate the peak between the two segments.
+                            self._resolve_hat_mesh_segment(index=ii - 1)
+
                         else:
+                            # adjust the length of the segment.
                             segment.s0 = legacy_mesh_segment_list[-1].s0
                             adjacent_segment.s1 = legacy_mesh_segment_list[-1].s0
                     else:
@@ -603,8 +696,8 @@ class Mesh:
 
                         if adjacent_segment.s0 != self.lower_bnd and adjacent_segment.is_fixed_length() \
                                 and adjacent_segment.has_decreasing_ds():
-                            if self.segment_list[i - 2].is_fixed_length() and \
-                                    self.segment_list[i - 2].has_increasing_ds():
+                            if self.segment_list[ii - 2].is_fixed_length() and \
+                                    self.segment_list[ii - 2].has_increasing_ds():
                                 '''
                                     Undershoot case mid-mesh. Calculate new intersection point. 
                                     Calling resolve gap again. 
@@ -615,12 +708,12 @@ class Mesh:
     
                                     '''
                                 # print("Recalculate resolve gap. ")
-                                self.segment_list[i - 2].s1 = adjacent_segment.s1
-                                self.segment_list[i - 2].var_ds_ratio = segment.DEFAULT_BG_REGION_RATIO
-                                self.segment_list[i - 2].ds1 = adjacent_segment.ds1
-                                self.remove_mesh_segment(i - 1)
-                                self.resolve_gap_mid_mesh(self.segment_list[i - 2], i - 2,
-                                                          ds_begin=self.segment_list[i - 2].ds0, ds_end=ds)
+                                self.segment_list[ii - 2].s1 = adjacent_segment.s1
+                                # self.segment_list[i - 2].var_ds_ratio = segment.DEFAULT_BG_REGION_RATIO
+                                self.segment_list[ii - 2].ds1 = adjacent_segment.ds1
+                                self.remove_mesh_segment(ii - 1)
+                                self.resolve_gap_mid_mesh(self.segment_list[ii - 2], ii - 2,
+                                                          ds_begin=self.segment_list[ii - 2].ds0, ds_end=ds)
 
                         if adjacent_segment.s0 == self.lower_bnd and self.periodic and \
                                 not adjacent_segment.has_increasing_ds() and \
@@ -642,10 +735,15 @@ class Mesh:
                             self._periodic_case()
                             legacy_mesh.phi_shift = self.phi_shift
 
-                i = -1
+                ii = -1
 
                 legacy_mesh.segment_list = numpy.array([], dtype=LegacyMeshSegment)
-            i += 1
+
+            ii += 1
+            jj += 1
+
+        if jj >= max_iterations:
+            raise Exception("Error: Legacy mesh failed to find a result. Try adjusting your mesh requirements. ")
 
         return legacy_mesh
 
@@ -725,6 +823,79 @@ class Mesh:
 
         return s, n0, n1, dsn0, dsn1, sh
 
+    def _check_resolve_gap_mid_mesh(self, index, ds_begin, ds_end):
+        """ This is identical to the function below "resolve_gap_mid_mesh" yet it does not perform any actions.
+        This is only for diagnostics. """
+
+        segment_begin = self.segment_list[index - 1]
+        segment_end = self.segment_list[index + 1]
+
+        ratio = self.segment_list[index].var_ds_ratio
+        s, n0, n1, dsn0, dsn1, sh = self.calculate_intersection(ds_begin=ds_begin, ds_end=ds_end,
+                                                                segment_begin_s1=segment_begin.s1,
+                                                                segment_end_s0=segment_end.s0,
+                                                                ratio=ratio)
+
+        if segment_begin.s1 < sh < segment_end.s0:
+            '''
+            CASE #1: the segment_begin.ds is fairly similar to segment.end_ds.
+                        |  |    |  |      |  |     |  |
+                        |  |    |  |  ==> |  | //\\|  |
+                        |==|    |==|      |==|// \\|==|
+            '''
+            nh0 = numpy.ceil(numpy.interp(sh, s, n0))
+            nh1 = numpy.ceil(numpy.interp(sh, s, n1))
+            dsh0 = numpy.interp(sh, s, dsn0)
+            dsh1 = numpy.interp(sh, s, dsn1)
+
+            ratio0 = numpy.power((dsh0 / ds_end), (1.0 / nh0))
+            ratio1 = numpy.power((dsh1 / ds_begin), (1.0 / nh1))
+
+            # Now make sure it works
+            n_test0 = self.get_n_test0(ratio0, segment_end.s0, sh, ds_end)
+            n_test1 = self.get_n_test1(ratio1, segment_begin.s1, sh, ds_begin)
+
+            if n_test0 == nh0 and n_test1 == nh1:
+                ''' divide the gap segment into two segments. One segment that raises to the peak and
+                another that connects the peak to segment_end.ds0'''
+                return dsh0, sh
+            else:
+                print("Error: n_test0 and n_test1 midmesh.")
+
+        if sh == segment_end.s0:  # end ds is much bigger than begin ds
+            '''
+            CASE #2: the segment_begin.ds is much smaller than segment.end_ds.
+                        |  |    |==|      |  |     |==|
+                        |  |    |  |  ==> |  |  // |  |
+                        |==|    |  |      |==|//   |  |
+            '''
+            nh1 = numpy.ceil(numpy.interp(sh, s, n1))
+            dsh1 = numpy.interp(sh, s, dsn1)
+            ratio1 = numpy.power((dsh1 / ds_begin), (1.0 / nh1))
+            # check it works
+            n_test1 = self.get_n_test1(ratio1, segment_begin.s1, sh, ds_begin)
+            if n_test1 == nh1:
+                return dsh1, sh
+            else:
+                print("ERROR: n_test1 mid-mesh.")
+
+        if sh == segment_begin.s1:
+            '''
+            CASE #3: the segment_begin.ds is much bigger than segment.end_ds.                  
+                        |==|    |  |      |==|     |  |
+                        |  |    |  |  ==> |  | \\  |  |
+                        |  |    |==|      |  |   \\|==|
+            '''
+            nh0 = numpy.ceil(numpy.interp(sh, s, n0))
+            dsh0 = numpy.interp(sh, s, dsn0)
+            ratio0 = numpy.power((dsh0 / ds_end), (1.0 / nh0))
+            # check it works
+            n_test0 = self.get_n_test0(ratio0, segment_end.s0, sh, ds_end)
+            if n_test0 == nh0:
+                return dsh0, sh
+            else:
+                print("ERROR: n_test0 mid-mesh.")
+
     def resolve_gap_mid_mesh(self, segment, index, ds_begin, ds_end):
         """
         Resolves the gap between two segments "segment_begin" and "segment_end" where the segment between the two is
