@@ -16,5 +16,5 @@ def plot_hdf(hdf_file_path):
 
 if __name__ == "__main__":
     # Choose the files to read, point the base_path to where you put the MapPipeline folder
-    hdf_file_path = '/Users/opalissan/Downloads/MagnetogramExamples/Oct2011/br_example_2011_10_01_raw.hdf'
+    hdf_file_path = "/Users/opalissan/Desktop/br_caplan_shine_fill_pt.h5"
     plot_hdf(hdf_file_path)
