@@ -329,7 +329,7 @@ class MeshApp:
                 elif j == 3 and rect.radial_var_ds is not None:
                     e.insert(tk.END, str(rect.radial_var_ds))
                 else:
-                    e.insert(tk.END, '')
+                    e.insert(tk.END, str(self.ui_session.RADIAL_FG_RATIO))
                 e.grid(row=10 + i, column=j + 1, sticky=tk.NSEW)
                 cols.append(e)
             self.radial_table_tb.append(cols)
@@ -568,7 +568,10 @@ class MeshApp:
         self.get_bg_ratio()
         self.get_fg_ratio()
         self.add_ds()
-        self.get_mesh_theta_1(show_plot)
+        try:
+            self.get_mesh_theta_1(show_plot)
+        except Exception:
+            print("Theta mesh failed. ")
 
     def get_mesh_theta_1(self, show_plot):
         """ include 4 steps of building a theta mesh. """
@@ -581,20 +584,24 @@ class MeshApp:
             if 0 <= box.t0 <= np.pi and 0 <= box.t1 <= np.pi:
                 mesh.insert_mesh_segment(MeshSegment(box.t0, box.t1, box.ds, var_ds_ratio=box.tp_var_ds))
         if mesh.is_valid():
+            input_mesh = mesh.json_dict()
             adj_mesh = mesh.resolve_mesh_segments().json_dict()
-            legacy_mesh = mesh.build_legacy_mesh().json_dict()
+            legacy_mesh = mesh.build_legacy_mesh(max_iterations=250).json_dict()
             mpl.pyplot.close("all")
             create_psi_mesh(adj_mesh, legacy_mesh, 't', self.mesh_results_folder, output_file_name="tmp_mesh_t.dat",
                             mesh_res_file_name="mesh_res_t.txt", show_plot=show_plot,
                             save_plot_path=self.mesh_results_folder,
-                            save_plot=True, plot_file_name="theta_mesh_spacing.png")
+                            save_plot=True, plot_file_name="theta_mesh_spacing.png", input_mesh=input_mesh)
 
     def phi(self, show_plot=True):
         """ generate a phi mesh."""
         self.get_bg_ratio()
         self.get_fg_ratio()
         self.add_ds()
-        self.get_mesh_phi_1(show_plot)
+        try:
+            self.get_mesh_phi_1(show_plot)
+        except Exception:
+            print("Phi mesh failed.")
 
     def get_mesh_phi_1(self, show_plot):
         """ include 4 steps of building a phi mesh. """
@@ -608,12 +615,14 @@ class MeshApp:
             mesh.insert_mesh_segment(MeshSegment(box.p0, box.p1, box.ds, var_ds_ratio=box.tp_var_ds))
 
         if mesh.is_valid():
+            input_mesh = mesh.json_dict()
             adj_mesh = mesh.resolve_mesh_segments().json_dict()  # adjusted mesh as a dictionary
-            legacy_mesh = mesh.build_legacy_mesh().json_dict()  # legacy mesh as a dictionary
+            legacy_mesh = mesh.build_legacy_mesh(max_iterations=250).json_dict()  # legacy mesh as a dictionary
             mpl.pyplot.close("all")
             create_psi_mesh(adj_mesh, legacy_mesh, 'p', self.mesh_results_folder, output_file_name="tmp_mesh_p.dat",
                             mesh_res_file_name="mesh_res_p.txt", show_plot=show_plot, save_plot=True,
-                            save_plot_path=self.mesh_results_folder, plot_file_name="phi_mesh_spacing.png")
+                            save_plot_path=self.mesh_results_folder, plot_file_name="phi_mesh_spacing.png",
+                            input_mesh=input_mesh)
         else:
             print("Phi input fails to build a mesh. Mesh is not valid.")
 
@@ -623,7 +632,10 @@ class MeshApp:
         self.get_radial_fg_ratio()
         self.get_radial_rMin()
         self.get_radial_rMax()
-        self.get_mesh_r_1(show_plot)
+        try:
+            self.get_mesh_r_1(show_plot)
+        except Exception:
+            print("Radial mesh failed. ")
 
     def read_radial_textbox_row(self, row, row_num):
         """ check if radial input textbox content is valid."""
@@ -667,12 +679,14 @@ class MeshApp:
                 mesh.insert_mesh_segment(MeshSegment(s0, s1, ds=ds, var_ds_ratio=var_ds))
             row_num += 1
         if mesh.is_valid():
+            input_mesh = mesh.json_dict()
             adj_mesh = mesh.resolve_mesh_segments().json_dict()
-            legacy_mesh = mesh.build_legacy_mesh().json_dict()
+            legacy_mesh = mesh.build_legacy_mesh(max_iterations=250).json_dict()
             mpl.pyplot.close("all")
             create_psi_mesh(adj_mesh, legacy_mesh, 'r', self.mesh_results_folder, output_file_name="tmp_mesh_r.dat",
                             mesh_res_file_name="mesh_res_r.txt", show_plot=show_plot, save_plot=True,
-                            save_plot_path=self.mesh_results_folder, plot_file_name="r_mesh_spacing.png")
+                            save_plot_path=self.mesh_results_folder, plot_file_name="r_mesh_spacing.png",
+                            input_mesh=input_mesh)
 
     def add_ds(self):
         """ check if ds textbox content in theta/phi are valid."""
@@ -826,7 +840,10 @@ class MeshApp:
                 i = len(self.ui_session.box_list) - 1
                 for j in range(4):
                     e = tk.Entry(self.rightFrame, width=5, bg='white')
-                    e.insert(tk.END, '')
+                    if j == 3:
+                        e.insert(tk.END, str(self.ui_session.RADIAL_FG_RATIO))
+                    else:
+                        e.insert(tk.END, "")
                     e.grid(row=10 + len(self.ui_session.box_list), column=j + 1, sticky=tk.NSEW)
                     cols.append(e)
                 self.radial_table_tb.append(cols)
